#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>

#include "timing.h"
#include "util.h"



enum {
    READ,
    WRITE
};



int Mode;                               // read or write
const char *FileName = "bench.dat";     // file to read/write
int FileSize;

int ActualTransferred;


dmtimer_t *RunningTimer;
double RunningTime;


unsigned int BufSize = 1048576;
unsigned char *Buf;                     // buffer for reading/writing.


void DoReadBenchmark() {

    int bytesread = 0;
    FILE *in = fopen( FileName, "rb" );

    if( in == NULL ) {
        printf( "error: couldn't open '%s' for read...\n", FileName );
        exit( 0 );
    }

    RunningTimer = StartTimer();

    while( (FileSize != -1 ? (bytesread < FileSize) : 1) && 
            fread( Buf, BufSize, 1, in ) ) {
        bytesread += BufSize;
    }

    RunningTime = (double)ReadTimer( RunningTimer ) / 1000.0;
    StopTimer( RunningTimer );

    ActualTransferred = bytesread;

}





void DoWriteBenchmark() {

    int byteswritten = 0;
    FILE *out = fopen( FileName, "wb" );

    if( out == NULL ) {
        printf( "error: couldn't open '%s' for write...\n", FileName );
        exit( 0 );
    }

    RunningTimer = StartTimer();

    while( byteswritten < FileSize && fwrite( Buf, BufSize, 1, out ) ) {
        byteswritten += BufSize;
    }

    RunningTime = (double)ReadTimer( RunningTimer ) / 1000.0;
    StopTimer( RunningTimer );

    ActualTransferred = byteswritten;

}






int main( int argc, const char *argv[] ) {

    int argn;

    if( argc < 3 ) {
        printf( "usage: dmbench {r|w} [b bufsize] [f filename] [s num_bytes]\n" );
        return( 0 );
    }
    
    if( argv[1][0] == 'r' ) {
        Mode = READ;
        FileSize = -1;
    } else {
        Mode = WRITE;
        FileSize = 100000000;
    }

    for( argn = 2; argn < argc; argn++ ) {
        switch( argv[argn][0] ) {
        case 'b':
            argn++;
            BufSize = atoi( argv[argn] );
            break;
        case 'f':
            argn++;
            FileName = argv[argn];
            break;
        case 's':
            argn++;
            FileSize = atoi( argv[argn] );
            break;
        }
    }
    

    printf( "dmbench -- %s '%s', %s byte%s...\n", 
        Mode == READ ? "reading" : "writing",
        FileName,
        FileSize == -1 ? "all the possible" : Fmt( "%d", FileSize ),
        FileSize == 1 ? "" : "s" );


    InitTime();

    Buf = malloc( BufSize );
    if( Buf == NULL ) {
        printf( "error: couldn't allocate %d bytes...\n", BufSize );
        return( 0 );
    }

    if( Mode == READ ) {
        DoReadBenchmark();
    } else {
        DoWriteBenchmark();
    }

    printf( "%d bytes in %f seconds\n", ActualTransferred, RunningTime );
    printf( "Raw performance: %f bytes per second (%f MB/s)\n", 
        (double)ActualTransferred / RunningTime, 
        (double)ActualTransferred / 1048576.0 / RunningTime );

    return( 0 );

}